//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs  3delite 2006-2009                                                                                   *
//*     See Filesystem Dialogs Readme.txt for details                                                                            *
//*                                                                                                                              *
//* Two licenses are available if you like this library:                                                                         *
//* Unlimited Shareware License: 100 Euros                                                                                       *
//* Single Commercial License: 950 Euros                                                                                         *
//*                                                                                                                              *
//*     http://www.shareit.com/product.html?productid=300070165                                                                  *
//*                                                                                                                              *
//*     http://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                           *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit FilesystemDialogsDynamic;

interface

Uses
    Windows;

const
    FILENAME_DLL_FD                       = 'FilesystemDialogs.dll';

    //* Exported function names
    NAME_FD_OpenDialog                    = 'FilesystemDialogs_OpenDialog';
    NAME_FD_SaveDialog                    = 'FilesystemDialogs_SaveDialog';
    NAME_FD_FolderDialog                  = 'FilesystemDialogs_FolderDialog';
    NAME_FD_Settings                      = 'FilesystemDialogs_SettingsWindow';

    NAME_FD_InitTOpenDialogParams         = 'FilesystemDialogs_InitTOpenDialogParams';
    NAME_FD_FreeTOpenDialogParams         = 'FilesystemDialogs_FreeTOpenDialogParams';
    NAME_FD_InitTOpenDialogResults        = 'FilesystemDialogs_InitTOpenDialogResults';
    NAME_FD_FreeTOpenDialogResults        = 'FilesystemDialogs_FreeTOpenDialogResults';

const
    //* Internal max. path size                       7800 bytes (30)
    FD_MAX_PATH                           = MAX_PATH * 30;

const

  {$EXTERNALSYM WM_USER}
  WM_USER             = $0400;

//* Filesystem Dialogs messages
const

  //* Pointer to TOpenDialogResults is in wParam
  //* Pointer to TOpenDialogParams is in lParam
  //* You get the messages in the hookproc and to the specified Parent window handle
  FDM_BASE              = 2 * WM_USER + 1;

  //* Filesystem Dialogs results message
  FDM_RESULTS           = FDM_BASE + 1;

  //* Load settings
  FDM_LOADSETTINGS      = FDM_BASE + 2;

  //* Free memory that you have allocated for the dialog('s params, results)
  FDM_FREEMEM           = FDM_BASE + 3;

  //* The dialog form is closing
  FDM_CLOSING           = FDM_BASE + 4;

  //* The dialog form's window handle in wParam
  //* Pointer to TOpenDialogParams is in lParam
  FDM_WINDOW_HANDLE     = FDM_BASE + 5;

  //* Realing the integrated form to the parent
  FDM_REALIGN_INTEGRATED_WINDOW = FDM_BASE + 6;

  //* Sent on file name selection
  //* Sent on file name selection
  //* File name is in wParam as PChar
  FDM_ON_SELECT_FILE_NAME = FDM_BASE + 7;

type
  TOpenOption = (ofReadOnly, ofOverwritePrompt, ofHideReadOnly, ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect, ofExtensionDifferent,
    ofPathMustExist, ofFileMustExist, ofCreatePrompt, ofShareAware, ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton, ofNoLongNames,
    ofOldStyleDialog, ofNoDereferenceLinks, ofEnableIncludeNotify, ofEnableSizing, ofDontAddToRecent, ofForceShowHidden, ofNotModal, ofNoClose,
    ofIntegrate, ofStayOnTop, ofNoCloseButton, ofOnSelectFilename);

  TOpenOptions = set of TOpenOption;

type
  OFNHookProc = function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT; stdcall;

type
  TOpenOptionEx = (ofExNoPlacesBar);
  TOpenOptionsEx = set of TOpenOptionEx;

//* Filesystem dialogs types
type
  TFSDialogType = (fsdtOpenDialog, fsdtSaveDialog, fsdtFolderDialog);

type
  TOFNOTIFY = packed record
    hdr: TNMHdr;
    lpOFN: Pointer; //tagOFN;
    pszFile: PWideChar;
  end;

//* Filesystem dialog Open file/Save file dialog Params
type
  TOpenDialogParams = record
    DefaultExt: PChar;          //* Default extension
    FileName: PChar;            //* Default file name for Open/Save and Initial dir
    Filters: PChar;             //* Filters eg.: 'Image files (*.bmp,*.jpg)|*.bmp;*.jpg|All files|*.*'
    FilterIndex: Integer;       //* Filter to use default, 1-first, 2-second...
    HelpContext: Integer;       //* Help context (not yet implemented)
    InitialDir: PChar;          //* Initial folder, this is extracted first from FileName, if no FileName specified settings are used to decide which folder to use
    Options: TOpenOptions;      //* Dialog options (not all implemented so far)
    OptionsEx: TOpenOptionsEx;  //* Extra Dialog options (not implemented, probably never will)
    Title: PChar;               //* Title to display as window's caption
    ParentWindow: HWND;         //* Parent window handle (needed for HookProc)
    DialogGUID: TGUID;          //* Specifically identificate the dialog (separate dialogs inside an application - you can get the same dialog always with this flag)
    HookProc: OFNHookProc;      //* Hookproc (There are two: 1. for not modal mode result, and 2. for filter change event (avoid using it) - ParentWindow handle is also needed)
    DialogType: TFSDialogType;  //* Type of the dialog: Open/Save/Folder
    AllowTIFFLZW: LongBool;     //* Allow LZW coded TIF image thumbnails (using these formats requires a license)
    //FileSize: Int64;			//* Estimated file size (will warn if won't fit in the selected directory before performing any saving)
    //MD5: String;				//* MD5 value of the file to save (compare to exisisting files)
    //User: Pointer;            //* User variable
  end;

//* Filesystem dialog result
type
  TOpenDialogResults = record
    Successfull: BOOL;          //* True: User pressed the Open/Save button or double-clicked on a file, False: User pressed Cancel or closed the window, or error (there is fallback to comdlg32.dll though)
    FileName: PChar;            //* The filename with full path the user selected (only one) ending with a double null: "C:\Dir\Data.txt" + #0#0
    FileNameSize: Int64;        //* FileName size in bytes (should be always at least MAX_PATH)
    FileCount: Cardinal;        //* Number of files, if user selected multiple files then FileCount > 1
    FileNames: PChar;           //* List of files in format "C:\Dir\Data.txt" + #0 + "D:\Folder\DataX.txt" separated by null characters ending with a double null
    FileNamesSize: Int64;       //* FileNames size in bytes (it's up to you to validate last entry if all selected filenames don't fit into the FileNames - but always check with FileExists())
    FilterIndex: Integer;       //* Filter's index the user selected 1-first, 2-second, etc...
  end;

//* Exported functions
type
  t_OpenDialog              = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
  t_SaveDialog              = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
  t_FolderDialog            = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;

  t_FilesystemDialogsSettings = function: BOOL; stdcall;

  t_InitTOpenDialogParams   = function(var Struct: TOpenDialogParams; MaxFiles: Int64 = 1000): Boolean; stdcall;
  t_FreeTOpenDialogParams   = function(var Struct: TOpenDialogParams): Boolean; stdcall;
  t_InitTOpenDialogResults  = function(var Res: TOpenDialogResults; MaxFiles: Int64 = 1000): Boolean; stdcall;
  t_FreeTOpenDialogResults  = function(var Struct: TOpenDialogResults): Boolean; stdcall;

var
    FilesystemDialogs_OpenDialog: t_OpenDialog;
    FilesystemDialogs_SaveDialog: t_SaveDialog;
    FilesystemDialogs_FolderDialog: t_FolderDialog;
    FilesystemDialogs_Settings: t_FilesystemDialogsSettings;
    FilesystemDialogs_InitTOpenDialogParams: t_InitTOpenDialogParams;
    FilesystemDialogs_FreeTOpenDialogParams: t_FreeTOpenDialogParams;
    FilesystemDialogs_InitTOpenDialogResults: t_InitTOpenDialogResults;
    FilesystemDialogs_FreeTOpenDialogResults: t_FreeTOpenDialogResults;

var
    FilesystemDialogsDLLHandle: THandle = 0;
    FilesystemDialogsDLLLoaded: Boolean = False;

    function InitFilesystemDialogsLibrary: Boolean;
    function FreeFilesystemDialogsLibrary: Boolean;

implementation

Uses
    SysUtils;

function InitFilesystemDialogsLibrary: Boolean;
begin
    FilesystemDialogsDLLHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));
    Result := FilesystemDialogsDLLHandle <> 0;
    if Result then begin
        FilesystemDialogs_OpenDialog            := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_OpenDialog));
        FilesystemDialogs_SaveDialog            := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_SaveDialog));
        FilesystemDialogs_FolderDialog          := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FolderDialog));
        FilesystemDialogs_Settings              := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_Settings));
        FilesystemDialogs_InitTOpenDialogParams := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_InitTOpenDialogParams));
        FilesystemDialogs_FreeTOpenDialogParams := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FreeTOpenDialogParams));
        FilesystemDialogs_InitTOpenDialogResults:= GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_InitTOpenDialogResults));
        FilesystemDialogs_FreeTOpenDialogResults:= GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FreeTOpenDialogResults));
    end;
    if (@FilesystemDialogs_OpenDialog = nil)
    OR (@FilesystemDialogs_SaveDialog = nil)
    OR (@FilesystemDialogs_FolderDialog = nil)
    OR (@FilesystemDialogs_Settings = nil)
    OR (@FilesystemDialogs_InitTOpenDialogParams = nil)
    OR (@FilesystemDialogs_FreeTOpenDialogParams = nil)
    OR (@FilesystemDialogs_InitTOpenDialogResults = nil)
    OR (@FilesystemDialogs_FreeTOpenDialogResults = nil)
        then Result := False;

    if Result
        then FilesystemDialogsDLLLoaded := True;
end;

function FreeFilesystemDialogsLibrary: Boolean;
begin
    Result := False;
    if FilesystemDialogsDLLHandle <> 0 then begin
        Result := FreeLibrary(FilesystemDialogsDLLHandle);
        if Result
            then FilesystemDialogsDLLLoaded := False;
    end;
end;

end.
