unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FilesystemDialogsDynamic, ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Edit3: TEdit;
    Label1: TLabel;
    CheckBoxOfIntegrate: TCheckBox;
    CheckBoxOfStayOnTop: TCheckBox;
    CheckBoxOfNoClose: TCheckBox;
    CheckBoxOfNotModal: TCheckBox;
    CheckBox1: TCheckBox;
    Edit4: TEdit;
    Label3: TLabel;
    Button8: TButton;
    Button12: TButton;
    Button7: TButton;
    Button1: TButton;
    Memo1: TMemo;
    PanelParent: TPanel;
    Edit1: TEdit;
    CheckBox2: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure Button12Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  fHookProc: OFNHookProc;

implementation

{$R *.dfm}


function OFNHookProc(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;
type
  POpenDialogResults = ^TOpenDialogResults;
var
  Res: POpenDialogResults;
  Text: String;
  Struct: Pointer;
begin

    //* Sent results
    if Msg = FDM_RESULTS then begin

        Res := POpenDialogResults(wParam);

        //* Extract first! filename
        Text := PChar(Res.FileName);

        Form1.Memo1.Lines.Add(Text);

    end;

    if Msg = FDM_ON_SELECT_FILE_NAME then begin

        Text := PChar(wParam);
        Form1.Memo1.Lines.Add(Text);

    end;

    //* Free the memory as it is not needed anymore
    if Msg = FDM_FREEMEM then begin

        Struct := Pointer(wParam);

        //* Free memory
        FilesystemDialogs_FreeTOpenDialogResults(TOpenDialogResults(Struct^));
    end;

end;

procedure TForm1.Button12Click(Sender: TObject);
var
  Success: Boolean;
begin

    //* Call FilesystemDialogs_SettingsWindow()
    Success := FilesystemDialogs_Settings;

    //* Use result
    Memo1.Clear;

    if Success
        then Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): OK.')
        else Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): ERROR.');

end;

procedure TForm1.Button1Click(Sender: TObject);
var
  pName: PChar;
  Success: Boolean;
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
begin

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.FileName := PChar(Edit1.Text);
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);

    //* Multi or single
    if CheckBox1.Checked
        then Params.Options := Params.Options + [ofAllowMultiSelect]
        else Params.Options := Params.Options - [ofAllowMultiSelect];

    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];

    //* Send a message when selection changes
    //* This requires a Handle which to the message will be sent and/or a HookProc
    if CheckBox2.Checked then begin
        Params.Options := Params.Options + [ofOnSelectFileName];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofOnSelectFileName];

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9C-F2ED-4BF7-B90E-0831BABA3093}');

    //* The app isn't commercial else you need a license
    Params.AllowTIFFLZW := True;

    ZeroMemory(@Results, SizeOf(Results));

    //* Inititalize the result structure (memory allocation)
    FilesystemDialogs_InitTOpenDialogResults(Results);

    //* Call OpenDialog()
    Success := FilesystemDialogs_OpenDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := Results.FileNames;
        while pName^ <> #0 do begin
            Memo1.Lines.Append(pName);
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);

    end;

    //* Free memory
    if NOT CheckBoxOfNotModal.Checked then begin

        FilesystemDialogs_FreeTOpenDialogResults(Results);

    end;

end;

procedure TForm1.Button7Click(Sender: TObject);
var
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
  pName: PChar;
  Success: Boolean;
begin

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.FileName := PChar(Edit1.Text);
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);
    Params.DialogType := fsdtSaveDialog;

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');

    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and a HookProc that will be invoked at the end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];

    //* Send a message when selection changes
    //* This requires a Handle which to the message will be sent and/or a HookProc
    if CheckBox2.Checked then begin
        Params.Options := Params.Options + [ofOnSelectFileName];
        Params.ParentWindow := PanelParent.Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofOnSelectFileName];

    //* The app isn't commercial else you need a license
    Params.AllowTIFFLZW := True;

    ZeroMemory(@Results, SizeOf(Results));

    //* Inititalize the result structure (memory allocation)
    FilesystemDialogs_InitTOpenDialogResults(Results);

    //* Call SaveDialog
    Success := FilesystemDialogs_SaveDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(Results.FileNames);

        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);

    end;

    //* Free memory
    if NOT CheckBoxOfNoClose.Checked then begin

        FilesystemDialogs_FreeTOpenDialogResults(Results);

    end;

end;

procedure TForm1.Button8Click(Sender: TObject);
var
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
  pName: PChar;
  Success: Boolean;
begin

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.InitialDir := PChar(ExtractFilePath(Edit1.Text));

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');

    ZeroMemory(@Results, SizeOf(Results));

    //* Inititalize the result structure (memory allocation)
    FilesystemDialogs_InitTOpenDialogResults(Results);


    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];

    //* Call FolderDialog
    Success := FilesystemDialogs_FolderDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(Results.FileNames);

    end;

    //* Free memory
    if NOT CheckBoxOfNotModal.Checked then begin

        FilesystemDialogs_FreeTOpenDialogResults(Results);

    end;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    InitFilesystemDialogsLibrary;
    if NOT FilesystemDialogsDLLLoaded then begin
        MessageDlg('Error loading ' + FILENAME_DLL_FD + '!', mtError, [mbOk], 0);
        Application.Terminate;
    end;

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    FreeFilesystemDialogsLibrary;
end;

end.
